unit FileFind;

interface

uses SysUtils, Forms, Classes, StdCtrls;

type
	TFileFoundEvent = procedure (Sender: TObject; Directory: String;
  	SearchRec: TSearchRec; var Stop : Boolean) of object;

  TFileFinder = class(TComponent)
  private
    { Private Declarations }
    FFileAttr: Integer;
    FRoot: String;
    FStop: Boolean;
    FOnFound: TFileFoundEvent;
    FSubFolders: Boolean;
    FProcessMsgs: Boolean;
    FFileFilter: String;

		procedure GetSubDirs(ARoot: String);
  protected
    { Protected Declarations }
  public
    { Public Declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Search;
    procedure Stop;
  published
    property FileAttr: Integer read FFileAttr write FFileAttr;
    property Root: String read FRoot write FRoot;
    property SubFolders: Boolean read FSubFolders write FSubFolders;
    property ProcessMessages: Boolean read FProcessMsgs write FProcessMsgs;
    property FileFilter: String read FFileFilter write FFileFilter;
    property OnFileFound: TFileFoundEvent read FOnFound write FOnFound;

  end;

procedure Register;

implementation

{--------------------------------------}
{ TFileFinder.Create                   }
{--------------------------------------}
constructor TFileFinder.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FRoot := 'C:\';
  FStop := False;
  FSubFolders := True;
  FFileAttr := faAnyFile;
	FProcessMsgs := True;
  FFileFilter := '*.*';

end;

{--------------------------------------}
{ TFileFinder.Destroy                  }
{--------------------------------------}
destructor TFileFinder.Destroy;
begin
  inherited Destroy;
end;

procedure TFileFinder.Search;
begin
	FStop := False;
	GetSubDirs(FRoot);
end;

procedure TFileFinder.Stop;
begin
	FStop := True;
end;

procedure TFileFinder.GetSubDirs(ARoot: String);
var
	Found: TSearchRec;
  Sub: String;
  i : Integer;
  Dirs: TStringList;
  Finished : Integer;
  iStop : Boolean;
begin
	iStop := FStop;
	Dirs := TStringList.Create;
	Finished := FindFirst(ARoot + '*.*', FFileAttr, Found);
  while (Finished = 0) and not (FStop or iStop) do
  begin
 		if (Found.Attr and faDirectory = faDirectory) and (Found.Name[1] <> '.') then
      	Dirs.Add(ARoot + Found.Name);
    if Assigned(FOnFound) then FOnFound(Self, ARoot, Found, iStop);
    FStop := iStop;
		Finished := FindNext(Found);
    if FProcessMsgs then Application.ProcessMessages;
  end;
  FindClose(Found);
	if FSubFolders and not (FStop or iStop) then
  	for i := 0 to Dirs.Count - 1 do
			GetSubDirs(Dirs[i] + '\');
  Dirs.Free;
end;

{--------------------------------------}
{ Register                             }
{--------------------------------------}
procedure Register;
begin
  RegisterComponents('Samples', [TFileFinder]);
end;

end.
